#include <windows.h>
#include <shobjidl.h> 

#include <string>
#include <vector>

std::string Utf8FromUtf16(const wchar_t* utf16_string);
std::wstring WStringFromString(const std::string& string);

/// <summary>
/// 
/// </summary>
/// <param name="description">
/// "filter description"
/// </param>
/// <param name="extensions">
/// "*.c; *.cpp"
/// </param>
/// <returns></returns>
std::string openFileDialog(std::string filterName, std::string filterPattern) {
    std::string path;

    HRESULT hr = CoInitializeEx(NULL, COINIT_APARTMENTTHREADED | COINIT_DISABLE_OLE1DDE);
    if (SUCCEEDED(hr)) {
        IFileOpenDialog* pFileOpen;

        // Create the FileOpenDialog object.
        hr = CoCreateInstance(CLSID_FileOpenDialog, NULL, CLSCTX_ALL,
            IID_IFileOpenDialog, reinterpret_cast<void**>(&pFileOpen));
        if (SUCCEEDED(hr)) {

            DWORD dwFlags;
            hr = pFileOpen->GetOptions(&dwFlags);
            if (SUCCEEDED(hr)) {

                hr = pFileOpen->SetOptions(dwFlags | FOS_FORCEFILESYSTEM);
                if (SUCCEEDED(hr)) {

                    std::wstring wName = WStringFromString(filterName);
                    std::wstring wPattern = WStringFromString(filterPattern);
                    COMDLG_FILTERSPEC fileTypes[] = {
                        { wName.c_str(), wPattern.c_str() },
                    };
                    hr = pFileOpen->SetFileTypes(ARRAYSIZE(fileTypes), fileTypes);
                    if (SUCCEEDED(hr)) {

                        // Show the Open dialog box.
                        HWND hwnd = GetActiveWindow();
                        hr = pFileOpen->Show(hwnd);
                        if (SUCCEEDED(hr)) {

                            // Get the file name from the dialog box.
                            IShellItem* pItem;
                            hr = pFileOpen->GetResult(&pItem);
                            if (SUCCEEDED(hr)) {
                                PWSTR pszFilePath;
                                hr = pItem->GetDisplayName(SIGDN_FILESYSPATH, &pszFilePath);

                                // set the file name.
                                if (SUCCEEDED(hr)) {
                                    path = Utf8FromUtf16(pszFilePath);
                                    CoTaskMemFree(pszFilePath);
                                }
                                pItem->Release();
                            }
                        }
                    }
                }
            }

            pFileOpen->Release();
        }

        CoUninitialize();
    }

    return path;
}

std::string Utf8FromUtf16(const wchar_t* utf16_string) {
    if (utf16_string == nullptr) {
        return std::string();
    }

    int target_length = ::WideCharToMultiByte(CP_UTF8, WC_ERR_INVALID_CHARS, utf16_string,
        -1, nullptr, 0, nullptr, nullptr);
    if (target_length == 0) {
        return std::string();
    }

    std::string utf8_string;
    utf8_string.resize(target_length);
    int converted_length = ::WideCharToMultiByte(
        CP_UTF8, WC_ERR_INVALID_CHARS, utf16_string,
        -1, utf8_string.data(), target_length, nullptr, nullptr);

    if (converted_length == 0) {
        return std::string();
    }

    return utf8_string;
}

std::wstring WStringFromString(const std::string& string) {
    std::vector<wchar_t> buff(
        MultiByteToWideChar(CP_ACP, 0, string.c_str(), (int)(string.size() + 1), 0, 0)
    );

    MultiByteToWideChar(CP_ACP, 0, string.c_str(), (int)(string.size() + 1), &buff[0], (int)(buff.size()));

    return std::wstring(&buff[0]);
}
